__author__= 'tmikkelsen'
import sys
import subprocess
import logging
import argparse
import os
import shutil
import ast

ENV_CACHE = '/tmp/env.cache'

class EnvironmentError(Exception):
    def __init__(self,message):
        self.message = message
        super(EnvironmentError,self).__init__(self.message)


class Envlib(object):
    def __init__(self,useFileCache=False):
        #Read in the current environment and break in to dict
        self.current = self.cacheVariables()
        self.fileCache = useFileCache
        if self.fileCache and os.path.isfile(ENV_CACHE):
            self.changes = dict()
            with open(ENV_CACHE,'r') as cache:
                self.changes = ast.literal_eval(cache.read())
                cache.close()
        else:
            self.changes = dict()

    def cacheVariables(self):
        attempts = 3
        self.valid = False
        while attempts > 0:
            try:
                output = subprocess.check_output(['fw_printenv'])
                variables = dict()
                for line in output.split("\n"):
                    index = line.find('=')
                    key = line[:index]
                    value = line[index+1:].rstrip()
                    if key != '':
                        variables[key] = value
                if len(variables) >= 5:
                    self.valid = True
                break
            except:
                self.valid = False
                attempts -= 1
                continue

        if self.valid:
            return variables

        #If we get to this point we have exhausted all three attempts to read
        #the environment, so bail out with an exception
        raise EnvironmentError("Unable to read the u-boot environment variables")
        

    def setenv(self,variable,value):
        self.changes[variable] = value
        if self.fileCache:
            with open(ENV_CACHE,'w') as cache:
                cache.write(str(self.changes))
                cache.close()

    def getenv(self,variable):
        try:
            return self.changes[variable]
        except:
            try:
                return self.current[variable]
            except:
                return False
    
    def deleteenv(self,variable):
        self.changes[variable] = '__DELETE__'
        if self.fileCache:
            with open(ENV_CACHE,'w') as cache:
                cache.write(str(self.changes))
                cache.close()

    def writeVariable(self,variable,value):
        try:
            if self.changes[variable] == '__DELETE__':
                subprocess.check_output(['fw_setenv','/dev/mtd6',variable])
            elif self.changes[variable] != '':
                subprocess.check_output(['fw_setenv','/dev/mtd6',variable,self.changes[variable]])
        except:
            print "Failed to update variable " + variable

    def saveenv(self):

        for variable in self.changes:
            if variable not in self.current or self.changes[variable] != self.current[variable]:
                self.writeVariable(variable,self.changes[variable])

        #Now verify that we still have all the variables we should

        failures = self.verify()
        attempts = 3
        while failures and attempts > 0:
            print "Verify failed"
            for variable in failures:
                self.writeVariable(variable,failures[variable])
            failures = self.verify()
            attempts = attempts - 1

        if self.fileCache:
            os.remove(ENV_CACHE)

        if failures:
            print "Failed to properly save the environment"
            return False
        
        return True

    def verify(self):

        #Build a verification dict that contains everything that existed before
        #as well as everything that was changed
        verify = self.current
        for variable in self.changes:
            verify[variable] = self.changes[variable]

        #Now get the variables as they stand now

        updated = self.cacheVariables()

        #Finally, test those variables against the verification list

        failures = dict()

        for variable in verify:
            if verify[variable] == '__DELETE__':
                #This was a deletion, make sure it happened
                if variable in updated:
                    #Didn't delete it, so note it again
                    failures[variable] = ''
            elif variable not in updated or verify[variable] != updated[variable]:
                failures[variable] = verify[variable]

        return failures 


    def initialized(self):
        return self.valid


def main():
    arg = argparse.ArgumentParser(description='M7 Environment Library')
    arg.add_argument('-g', help='Get the value of a variable',nargs=1)
    arg.add_argument('-s', help='Set the value of a variable in the cache',nargs=2)
    arg.add_argument('-d', help='Delete a variable',nargs=1)
    arg.add_argument('-w', help='Write the environment cache to flash and verify',action='store_true')

    args= arg.parse_args()
    
    env = Envlib(useFileCache=True)
    if not env.initialized():
        return 1

    if args.g:
        result = env.getenv(args.g[0])
        if result:
            print result
            return 0
        else:
            return 1
    if args.s:
        if env.setenv(args.s[0],args.s[1]):
            return 0
        else:
            return 1
    if args.d:
        if env.deleteenv(args.d[0]):
            return 0
        else:
            return 1

    if args.w:
        if env.saveenv():
            return 0
        else:
            return 1


if __name__ == '__main__':
    sys.exit(main())
